<?php

require_once 'base_controller.php';

/**
 * Controller for configuring PNP Pages, known as "Graph Collections" in our UI
 */
class pnp_pages_controller extends base_controller {
	public $name = 'Graph Collections';

	function __construct($input) {
		if (!in_array(NACOMA_ACCESS_FULL_ACCESS, get_nacoma_access('pnp'))) {
			deny_access("You need full 'access_rights' permissions to modify Graphs.");
		}
		$this->input = $input;
	}

	/**
	 * List PNP pages
	 */
	function index() {
		global $page;
		$objects = pnp_page_class::get_saved_pages();
		ksort($objects);
		$objects['new'] = new pnp_page_class();
		$description = <<<DESC
<p>
	Graph Collections are, of course, collections of Graphs. You decide which graphs that should be included in a collection
	through the selectboxes and text inputs below.
</p>

<p>
	If you wish to include graphs by using patterns of your services' names, you can use regular expressions. The official documentation includes <a target="blank" href="http://docs.pnp4nagios.org/pnp-0.6/pages">more information about regular expressions and collections in general</a>.
</p>

<p>
	When you have configured your Graph Collections, they will immidiately be visible in the Graphs-GUI; just look for this icon in the Graphs' navigation bar: <img src='/monitor/op5/pnp/media/images/pages.png' alt='' style='width: 32px; height: 32px; vertical-align: middle' />. This also means that you do not need to apply your changes when dealing with Graph gollections, they are generated as soon as you save them.
</p>
DESC;
		global $page;
		$input = $this->input;
		require_once dirname(__FILE__).'/../../templates/many-per-page.php';
	}

	/**
	 * Delete a PNP page
	 */
	function delete() {
		global $page;
		try {
			$pnp_page = pnp_page_class::from_id($_REQUEST['obj_id']);
			$pnp_page->delete_object();
			$page->print_message("<p>Successfully deleted '".$_REQUEST['obj_id']."'</p>");
		} catch(RuntimeException $e) {
			$page->print_validation_error_table(new pnp_page_class, array($e->getMessage()));
		}
		$input = $this->input;
		// @todo redirect
		$this->index();
	}

	/**
	 * Handle submitted data
	 */
	function submit() {
		global $page;
		$input = $this->input;
		$errors = array();
		$success_stories = array();
		$duplicate_names = array();
		foreach($_REQUEST['pnp_page'] as $old_name => $definition) {
			// if all fields are empty, screw saving a new one
			if('new' == $old_name && !$definition['page_name'] && !$definition['host_name'] && !$definition['service_desc']) {
				continue;
			}
			if(in_array($definition['page_name'], $duplicate_names)) {
				if('new' == $old_name) {
					$errors[$old_name] = sprintf("<p>Can not use the name '%s' because it is already taken.</p>", $definition['page_name']);
				} else {
					$errors[$old_name] = sprintf("<p>Can not change name of '%s' because the new name '%s' is already taken.</p>", $old_name, $definition['page_name']);
				}
				continue;
			}
			$duplicate_names[] = $definition['page_name'];
			$definition['file'] = $definition['page_name'];
			try {
				$pnp_page = new pnp_page_class($definition);
				$pnp_page->save_object();
				if($old_name != $definition['page_name'] && 'new' != $old_name) {
					// there was a change of names, in this stage
					// the new file is saved but the old one
					// also still exists. O(n) for removing each old
					// copy
					pnp_page_class::delete_file($old_name);
					$success_stories[] = sprintf("<p>Successfully saved Graph Collection definitions for '%s' (previously '%s')</p>", $definition['page_name'], $old_name);
				} else {
					$success_stories[] = sprintf("<p>Successfully saved Graph Collection definitions for '%s'</p>", $definition['page_name']);
				}
			} catch (Exception $e) {
				$errors[$old_name] = $e->getMessage();
			}
		}
		if($errors) {
			// just grab the latest $pnp_page since it's not
			// actually used in this following method
			$page->print_validation_error_table(new pnp_page_class, $errors);
		}
		if($success_stories) {
			$page->print_message($success_stories);
		}
		$this->index();
	}
}
