<?php

/**
 * PNP Page objects
 */
class pnp_page_class extends object_class {
	// @todo wow this should so not be hardcoded
	private static $page_dir = "/opt/monitor/etc/pnp/pages/";
	private static $saved_pages = array();

	// stuff needed for inheritance compliance
	public $properties = array(
		'var_index' => array(
			'use_regex' => 'pnp_page_use_regex',
			'page_name' => 'string',
			'host_name' => 'string', // should be host_regex
			'service_desc' => 'string', // should be service_regex
			'services' => 'pnp_page_service_mselect' // should be services
		),
		'required' => array(
			'use_regex',
			'page_name'
		),
		'label' => array(
			'use_regex' => 'Use regex',
			'page_name' => 'Page name',
			'host_name' => 'Host name',
			'service_desc' => 'Service description',
			'hosts' => 'Hosts',
			'services' => 'Services'
		),
		// special = [var_index name] => object type (eg. pnp_page or host)
		'special' => array(
			'services' => 'service'
		),
		'topic' => 'page_name'
	);
	public $type = 'pnp_page';
	public $graphs = array(); /**< graphs is a service oriented array, [] => [service] => hosts[] */

	public static $available_graphs = array(); /**< See get_available_graphs */

	const PERFDATA_DIR = "/opt/monitor/op5/pnp/perfdata/"; /**< Location of PNP perfdata */

	/**
	 * @param $options = array()
	 */
	function __construct($options = array()) {
		if($options) {
			$this->assign($options);
		}
	}

	/**
	 * Traverses given location of pnp perfgraphs whereabouts.
	 *
	 * No confirmation of the file's contents are done, with respect to performance.
	 *
	 * @todo consider traversing all of them, depending on how it should scale
	 *
	 * @param $host string
	 * @param $service string
	 * @param $use_regex = false boolean
	 * @return boolean
	 */
	private static function _graph_exists($host, $service, $use_regex = false) {
		$graphs_that_do_exist = self::get_available_graphs();
		if($use_regex) {
			if($host_matches = preg_grep("~$host~", array_keys($graphs_that_do_exist))) {
				foreach($host_matches as $matching_host) {
					foreach(array_keys($graphs_that_do_exist[$matching_host]) as $existing_service) {
						if(preg_match("~$service~", $existing_service)) {
							return true;
						}
					}
				}
			}
		} else if(isset($graphs_that_do_exist[$host]) && isset($graphs_that_do_exist[$host]["$host;$service"])) {
			return true;
		}
		return false;
	}

	/**
	 * Native pnp pages config into options array
	 *
	 * Nearly almost copy pasted from pnp's Data_Model::parse_page_cfg()
	 *
	 * @param $data array lines of config file
	 * @return array
	 */
	private static function _config_to_options($data) {
		$inside=0;
		$allowed_tags = array("page", "graph");
		$options = array();
		$l = 0;
		foreach($data as $line){
			if(preg_match('/(^#|^;)/',$line)) {
				continue;
			}

			preg_match('/define\s+(\w+)\W+{/' ,$line, $tag);
			if(isset($tag[1]) && in_array($tag[1],$allowed_tags)){
				$inside = 1;
				$t = $tag[1];
				$l++;
				continue;
			}
			if(preg_match('/\s?(\w+)\s+([^#]+)(#.*)?$/',$line, $key) && $inside == 1){
				$k=strtolower($key[1]);
				$v=$key[2];
				if($t=='page'){
					$options[$k] = trim($v);
				}elseif($t=='graph'){
					$options[$l][$k] = trim($v);
				}
			} elseif(preg_match('/}/',$line)){
				$inside=0;
				$t = "";
				continue;
			}
		}
		return $options;
	}

	/**
	 * Setup object to conform to the Nacoma standard, see the public variables
	 * of object_class for more information
	 *
	 * @param $options array
	 * @return pnp_page_class
	 * @throws RuntimeException if graphs doesn't exist
	 */
	function assign(array $options) {
		$file = pathinfo($options['file'], PATHINFO_BASENAME);
		$page_name = str_replace(' ', '_', trim(preg_replace('/\.cfg$/', null, $file)));
		$use_regex = 1 == isset($options['use_regex']) && $options['use_regex'] ? 1 : 0;
		if(1 == $use_regex) {
			if(isset($options['host_name'])) {
				// we got POST:ed
				$this->obj['host_name'] = $options['host_name'];
				$this->obj['service_desc'] = $options['service_desc'];
			} else {
				// we're reading a saved config and we want the graph part
				// i.e. the only numeric index in that array
				foreach($options as $key => $values) {
					if(!is_numeric($key)) {
						continue;
					}
					$this->obj['host_name'] = $values['host_name'];
					$this->obj['service_desc'] = $values['service_desc'];
				}
			}
		} elseif(isset($options['services'])) {
			// we're getting results from the multiselect (POST)

			// graphs is a service oriented array, [] => [service] => hosts[]
			$this->graphs = array();
			$services = array();
			foreach($options['services'] as $service_key => $service) {
				preg_match("~(?P<host>.+);(?P<service>.+)~", $service, $parts);
				$host = $parts['host'];
				$service = $parts['service'];
				if(self::_graph_exists($host, $service, $use_regex)) {
					if(!isset($this->graphs[$service])) {
						$this->graphs[$service] = array();
					}
					$this->graphs[$service][] = $host;
					$services[] = "$host;$service";
				}
			}
			// we want this to be able to give mselect some selected values
			$this->obj['services'] = implode(',', $services);
		} else {
			// we're getting results from a saved config file
			$services = array();
			foreach($options as $option => $specification) {
				if(!is_numeric($option)) {
					// we hit an option that is not a graph, continue
					continue;
				}
				foreach(explode(',', $specification['service_desc']) as $service) {
					$this->graphs[$service] = array();
					foreach(explode(',', $specification['host_name']) as $host) {
						$services[] = "$host;$service";
						$this->graphs[$service][] = $host;
					}
				}
			}
			$this->obj['services'] = implode(',', $services);
		}
		$this->obj['use_regex'] = $use_regex;
		$this->obj['page_name'] = $this->id = $page_name;
		return $this;
	}

	/**
	 * @param $filename string
	 * @return boolean
	 * @throws RuntimeException
	 */
	static function delete_file($filename) {
		$filename = self::$page_dir.preg_replace("/\.cfg$/", null, $filename).'.cfg';
		if(!file_exists($filename)) {
			return true;
		}
		if(!is_writable($filename)) {
			throw new RuntimeException("Not enough permissions to delete the file ".$filename);
		}
		if(!unlink($filename)) {
			throw new RuntimeException("Not able to delete the file ".$filename);
		}
		return true;
	}

	/**
	 * @return boolean
	 * @throws RuntimeException
	 */
	function delete_object() {
		if(!$this->id) {
			throw new RuntimeException("You should use ".__CLASS__."::assign() prior to deleting a pnp_pages object");
		}
		$file = preg_replace("/\.cfg$/", null, self::$page_dir.$this->obj['page_name']).'.cfg';
		if(!file_exists($file)) {
			throw new RuntimeException("The file '".$file."' does not exist");
		}
		if(!is_writable($file)) {
			throw new RuntimeException("I don't have sufficient rights for removing the file ".$file);
		}
		if(!unlink($file)) {
			throw new RuntimeException("Tried to remove '$file' and failed even though the permissions are correct");
		}
	}

	/**
	 * Factory method for translating a file to an object
	 *
	 * @param $filename string
	 * @return pnp_page_class
	 * @throws RuntimeException
	 */
	static function from_file($filename) {
		if(!is_readable($filename)) {
			throw new RuntimeException("Unable to read '$filename'");
		}
		$config = self::_config_to_options(file($filename));
		$config['file'] = $filename;
		$instance = new pnp_page_class($config);
		return $instance;
	}

	/**
	 * A page's name (accepts both $name and $name.cfg)
	 *
	 * @param $id string
	 * @return pnp_page_class
	 * @throws RuntimeException
	 */
	static function from_id($id) {
		$id = preg_replace('/\.cfg$/', null, $id).'.cfg';
		return self::from_file(self::$page_dir.$id);
	}

	/**
	 * Expensive function, traverses all perfdata directories and collects filenames
	 *
	 * @return array [string host] => [string service] => string filename (full path)
	 */
	static function get_available_graphs() {
		if(self::$available_graphs) {
			return self::$available_graphs;
		}
		foreach(glob(pnp_page_class::PERFDATA_DIR."*/*.xml") as $filename) {
			$parts = explode('/', $filename);
			$service = preg_replace("/\.xml$/", null, array_pop($parts));
			$host = array_pop($parts);
			$service = "$host;$service";
			if(isset(self::$available_graphs[$host]) && is_array(self::$available_graphs[$host])) {
				self::$available_graphs[$host][$service] = $filename;
			} else {
				self::$available_graphs[$host] = array($service => $filename);
			}
		}
		return self::$available_graphs;
	}

	/**
	 * Make sure the controller is used instead of the generic delete.php
	 *
	 * @return array
	 */
	function get_cmd_bar_links() {
		$links = array();
		if ($this->id !== 'new' && !empty($this->id)) {
			$links['delete'] = "<a title='Delete this object' class='delete' href=\"pnp_pages?action=delete&amp;obj_id=$this->id\">";
		}
		return $links;
	}

	/**
	 * @return array [config_name] => pnp_page_class
	 */
	static function get_saved_pages() {
		if(self::$saved_pages) {
			return self::$saved_pages;
		}
		$page_configs = glob(self::$page_dir.'*.cfg');
		foreach($page_configs as $page_config) {
			try {
				self::$saved_pages[pathinfo($page_config, PATHINFO_BASENAME)] = self::from_file($page_config);
			} catch(Exception $e) {
				// log badly formatted configs? i.e. edited by hand and never used
			}
		}
		return self::$saved_pages;
	}

	/**
	 * @return string
	 */
	function get_object_name() {
		return $this->obj['page_name'] ? $this->obj['page_name'] : 'New Collection';
	}

	/**
	 * Does not write to db, write to file
	 *
	 * @param $user = true boolean
	 * @return string full path of newly created page
	 * @throws RuntimeException
	 */
	function save_object($user=true) {
		if(!$this->id) {
			throw new RuntimeException("You should use ".__CLASS__."::assign() prior to saving a pnp_pages object");
		}
		return $this->write_to_file($this->obj['page_name']);
	}

	/**
	 * Write current properties to file
	 *
	 * @param $filename string
	 * @return string new filename
	 * @throws RuntimeException couldn't write to given filename
	 */
	function write_to_file($filename) {
		$use_regex = 1 == isset($this->obj['use_regex']) && $this->obj['use_regex'] ? 1 : 0;
		$content = "#NACOMA
define page {
	use_regex ".$use_regex."
	page_name ".$this->obj['page_name']."
}";

		if($use_regex) {
			$content .= "
define graph {
	host_name ".$this->obj['host_name']."
	service_desc ".$this->obj['service_desc']."
}";
		} elseif(isset($this->graphs)) {
			// we've got values from selectboxes, N:N between services and hosts
			$graphs = array();
			foreach($this->graphs as $service => $hosts) {
				$graphs[] = "
define graph {
	host_name ".implode(',', $hosts)."
	service_desc $service
}";
			}
			$content .= implode("\n", $graphs);
		}
		$filename = preg_replace('/\.cfg$/', null, $filename).'.cfg';
		$full_path = "/opt/monitor/etc/pnp/pages/$filename";
		if(!file_put_contents($full_path, $content, LOCK_EX)) {
			throw new RuntimeException("Could not write to $full_path, please check the permissions");
		}
		return $full_path;
	}
}
