<?php

/**
 * A whole class dedicated only to that tiny thing at the top of the page. Wow!
 *
 * Sort of silly as a whole class, but we need to be able to add warning
 * messages during rendering, so it's better to have all the messages in one
 * place so they don't go out of sync.
 */
class top_bar_class {
	private static $too_many_hosts = "Maximum number of hosts exceeded! Current hosts: %d. Max allowed: %d.";
	private static $almost_too_many = "License usage over 95%% (Using %d of %d). When the license is exceeded, configuration can no longer be saved.";
	private static $unsaved_changes = 'You have unsaved configuration changes. Please press SAVE to verify and activate.';
	private static $role_tryout = "This is the '%s' role. Click <a href=\"edit_special.php/roles?action=end_tryout\" style=\"text-decoration: underline;\">here</a> to use your regular role.";

	private static function is_in_tryout() {
		return isset($_SESSION['tryout_role_id']);
	}

	private static function get_tryout_role_name() {
		if (isset($_SESSION['tryout_role_id'])) {
			$res = pdo_exec_query('SELECT role_name FROM role WHERE id = '.pdo_quote_string($_SESSION['tryout_role_id']));
			$row = $res->fetch(PDO::FETCH_NUM);
			if ($row === false) {
				unset ($_SESSION['tryout_role_id']);
			} else {
				$role_name = $row[0];
				return $role_name;
			}
		}
		return false;
	}

	/**
	 * Render the top bar
	 */
	public static function render() {
		$allowed = get_num_hosts_allowed();
		$configured = get_num_hosts_configured();

		$to_show = array();
		$ret = false;

		if ($configured != 0 && $allowed != 0) {
			if ($configured > $allowed)
				$to_show[] = '<span id="too_many">'.sprintf(self::$too_many_hosts, $configured, $allowed).'</span>';
			else if ($configured > $allowed * 0.95)
				$to_show[] = '<span id="too_many">'.sprintf(self::$almost_too_many, $configured, $allowed).'</span>';
		}

		if ((bool) Changelog::get_changes_count())
			$to_show[] = '<span id="unsaved">'.self::$unsaved_changes.'</span>';

		if (self::is_in_tryout()) {
			$role_name = self::get_tryout_role_name();
			$to_show[] = '<span id="in_tryout">'.sprintf(self::$role_tryout, $role_name).'</span>';
		}

		if (empty($to_show))
			return '<div class="config-warning"></div>';

		$ret .= '<div class="config-warning">';
		if (count($to_show) === 1)
			$ret .= '<p style="padding-top: 6px;">'.$to_show[0].'</p>';
		else
			$ret .= '<p>'.implode($to_show).'</p>';
		$ret .= '</div>';

		return $ret;
	}

	/**
	 * Add warning to the top-bar that there are changed but
	 * unsaved objects in the database
	 */
	public static function add_unsaved_warning() {
		if ((bool) Changelog::get_changes_count()) {
			?>
			<script type="text/javascript">
				$('.config-warning #unsaved').remove();
				if ($('.config-warning p').length == 0)
					$('.config-warning').append('<p></p>');

				$('.config-warning p').append('<span id="unsaved"><?php echo self::$unsaved_changes; ?></span>');
				if ($('.config-warning span').length > 1)
					$('.config-warning p').attr('style', '');
				else if ($('.config-warning span').length == 1)
					$('.config-warning p').attr('style', 'padding-top: 6px');
			</script>
			<?php
		}
	}

	/**
	 * Adds warning to the top-bar with the supplied message.
	 */
	public static function add_warning($msg) {
		?>
		<script type="text/javascript">
			$('.config-warning #add_warning').remove();
			if ($('.config-warning p').length == 0) {
				$('.config-warning').append('<p></p>');
			}

			$('.config-warning p').append('<span id="add_warning"><?php echo $msg; ?></span>');
			if ($('.config-warning span').length > 1)
				$('.config-warning p').attr('style', '');
			else if ($('.config-warning span').length == 1)
				$('.config-warning p').attr('style', 'padding-top: 6px');
		</script>
		<?php
	}
}
