<?php

/**
 * Handle input from clone.php by calling createProfile() and
 * createSingleProfileObject() to save as profile.
 *
 * 	@param $obj Object to use as profile
 * 	@param $profile_name Name of profile to create
 * 	@param $profile_description Selfexplanatory
 * 	@param $sub_objects_to_clone Array of subobjects to clone
 */
function saveAsProfile($obj,  $profile_name='', $profile_description='', $sub_objects_to_clone=false)
{
	global $obj_class, $page, $Object_Index_Table;

	$obj_type 				= $obj->type;
	$obj_id					= $obj->id;
	$profile_name 			= trim(addslashes($profile_name));
	$profile_description 	= trim(addslashes($profile_description));
	$profile_arr			= false;
	$profile_id				= 0 ;
	$obj_name				= '';
	$master_info			= false;
	$sub_arr				= false;

	get_object_index_table('host');

	if ($obj_type=='' || !$sub_objects_to_clone){
		return false;
	}
	$fname = 'classes/'.$obj_class.'.php';
	if (!is_file($fname)) {
		echo "Missing class file '".$fname."'. Bailing out\n";
		$page->html_end();
	}
	require_once($fname);
	$class = new $obj_class();

	// Get info on cloned master object
	$obj_info = $class->getObjectById($obj_id, $obj_type);
	switch ($obj_type){
		case 'host':
			$obj_name = $obj_info['host_name'];
			break;
		case 'service':
			$obj_name = $Object_Index_Table['host'][$obj->master_obj_id];
			//$obj_info['service_description'];
			break;
	}

	// Create profile instance
	$profile_id = createProfile($obj_type, $obj_name, $profile_name, $profile_description);
	if (!$profile_id){
		return false;
	}

	// Loop through sub objects and decide what to save
	foreach ($sub_objects_to_clone as $sub_type => $sub_obj){
		if (is_array($sub_obj)){
			foreach ($sub_obj  as $id => $obj){
				foreach ($obj as $type => $on_off){
					if ($on_off=='on'){
						@$sub_arr[$id]['bits'] = (int)$sub_arr[$id]['bits'] + translateInfoToBit($type);

						if (@$sub_arr[$id]['name']==''){
							$sub_info = $class->getObjectById($id, $sub_type);
							$sub_name_field 		= getObjectNameField($sub_type);
							$sub_arr[$id]['name'] 	= $sub_info[$sub_name_field];
							$sub_arr[$id]['type']	= $type;
						}
					}
				}

			}
		} else {
			// info on master object
			if ($sub_obj=='on'){
				$bit = translateInfoToBit($sub_type);
				$master_info += $bit;
			}
		}
	}

	// insert master_object
	switch ($obj_type){
		case 'host':
			createSingleProfileObject($profile_id, $obj_name, '', $master_info);
			break;
		case 'service':
			createSingleProfileObject($profile_id, $obj_name, '', $master_info);
			break;
	}

	// insert sub objects
	if (is_array($sub_arr)){
		foreach ($sub_arr as $key => $val){
			if ($val['type']=='service'){
				createSingleProfileObject($profile_id, '', $val['name'], $val['bits']);
			} else {
				echo "Objects of type ".$val['type']." are not supported...";
			}
		}
	} else {
		createSingleProfileObject($profile_id, '', $obj_info['service_description'], $master_info);
	}
	return $profile_name;
}

/**
 * Transform <object_type>extinfo, <object_type>escalation,
 * <object_type>dependency to bit value
 * 	@param $type name of object type
 * 	@return int
 */
function translateInfoToBit($type='')
{
	if (strstr($type, 'extinfo')) return 1;
	if (strstr($type, 'escalation')) return 2;
	if (strstr($type, 'dependency')) return 4;
}

/**
 * Transform bit values back to array
 * @param $bit Integer used as bitmask
 * @return Array of names with settings
 */
function translateBitToInfo($bit=0)
{
	$retval = array();
	if (!$bit) return 0;
	switch ($bit) {
		case 1:
			$retval['extinfo'] 		= 1;
			$retval['escalation'] 	= 0;
			$retval['dependency'] 	= 0;
			break;
		case 2:
			$retval['extinfo'] 		= 0;
			$retval['escalation'] 	= 1;
			$retval['dependency'] 	= 0;
			break;
		case 3:
			$retval['extinfo'] 		= 1;
			$retval['escalation'] 	= 1;
			$retval['dependency'] 	= 0;
			break;
		case 4:
			$retval['extinfo'] 		= 0;
			$retval['escalation'] 	= 0;
			$retval['dependency'] 	= 1;
			break;
		case 5:
			$retval['extinfo'] 		= 1;
			$retval['escalation'] 	= 0;
			$retval['dependency'] 	= 1;
			break;
		case 6:
			$retval['extinfo']		= 0;
			$retval['escalation'] 	= 1;
			$retval['dependency'] 	= 1;
			break;
		case 7:
			$retval['extinfo'] 		= 1;
			$retval['escalation'] 	= 1;
			$retval['dependency'] 	= 1;
			break;
	}
	return $retval;
}

/**
 * Get field name in db depending on object_type
 * @param $obj_type name of object type
 * @return string
 */
function getObjectNameField($obj_type='')
{
	if ($obj_type==''){
		return false;
	}

	switch ($obj_type){
		case 'host':
			$obj_name = 'host_name';
			break;
		case 'service':
			$obj_name = 'service_description';
			break;
	}
	return $obj_name;
}

/**
 * Create record in prototypes which will act as
 * 'handle' for all sub objects
 * 	@param 	str obj_type
 * 	@param 	str obj_name
 * 	@param 	str profile_name [optional]
 * 	@param 	str profile_description [optional]
 * 	@return int
 */
function createProfile($obj_type, $object_name='', &$profile_name='', $profile_description='')
{
	$obj_type 				= trim(addslashes($obj_type));
	$profile_name 			= trim(addslashes($profile_name));
	$object_name 			= trim(addslashes($object_name));
	$profile_description 	= trim(addslashes($profile_description));
	if ($profile_name==''){
		$profile_name = ucfirst($obj_type)." profile, ".date('Ymd');
	}
	if ($obj_type==''){
		return false;
	}
	$sql = "INSERT INTO profiles(name, object_type, object_name, info) VALUES('".$profile_name."', '".$obj_type."', '".$object_name."', '".$profile_description."')";
        try
        {
            $res = pdo_exec_query($sql);
        }
        catch(Exception $ex) {
		return false;
	}
	return pdo_last_insert_id('profiles');
}

/**
 * Insert single sub object for profile
 * @param $profile_id Integer
 * @param $host_name host_name
 * @param $service_description service description
 * @param $sub_objects integer deciding if we should clone subobjects
 */
function createSingleProfileObject($profile_id=0, $host_name='', $service_description='', $sub_objects=0)
{
	$profile_id 	= (int)$profile_id;
	$sub_objects	= (int)$sub_objects;

	if (!$profile_id)
	  return false;

	$sql = "INSERT INTO profile_objects(profile_id, host_name, service_description, sub_objects) VALUES(".$profile_id.", '".$host_name."', '".$service_description."', ".$sub_objects.")";
        try {
            $res = pdo_exec_query($sql);
        }
        catch(Exception $ex) {
            echo pdo_error_message();
            return false;
	}
	return true;
}

/**
 * Retrieve all info on a profile from db
 * @param $id Id number of the profile
 * @return  profile['info']=str, profile['type']=str, profile['name']=str,
 * 			profile['sub_objects']=array('obj_type', 'object_id', 'description')
 */
function fetchProfile($id=0)
{
	$id 			= (int)$id;
	$sub_objects 	= false;
	$sub_type		= false;
	$master_id		= 0;
	$master_object_id = 0;
	$master_info	= false;

	if (!$id){
		return false;
	}

	$sql = "SELECT * FROM profiles WHERE id=".$id;
	$res = pdo_exec_query($sql);
        $profile = pdo_fetch_array($res);
        unset($res);
	if (!$profile){
		return false;
	}
	$type 		= $profile['object_type'];
	$profile_id	= $profile['id'];
	require_once('classes/'.$type.'_class.php');
	$class_name	= $type.'_class';
	$class = new $class_name();

	// get original objects data
	$master_info 	= getObjectByName($profile['object_name'], 'host');
	$master_id		= $master_info['id'];

	switch ($type){
		case 'host':
			$query = "SELECT id, service_description AS the_name FROM service WHERE host_name=".$master_id;
			break;
		case 'service':
			$master_object_id = $master_id;
			$query = "SELECT s.id, s.service_description AS the_name FROM service s, profile_objects po WHERE s.host_name=".$master_id." AND s.service_description=po.service_description AND po.profile_id=".$profile_id;
			break;
	}

	// get profile_objects data
	$sub_objects_res = pdo_exec_query($query);
        $count = 0;
	while ($sub_object_row = $sub_objects_res->fetch(PDO::FETCH_ASSOC)){
            ++$count;
		//echo $sub_object_row['id']."<br />";
		$sub_objects['service'][$sub_object_row['the_name']] = $sub_object_row['id'];
		if ($type=='service'){
			$master_id = $sub_object_row['id'];
			//print_r($sub_object_row);
		}
	}
        unset($sub_objects_res);
        if( 0 == $count ) {
            return false;
        }
	//print_r($sub_objects);
	if ($sub_objects){
		$return['info'] 		= $profile['info'];
		$return['type'] 		= $profile['object_type'];
		$return['name'] 		= $profile['name'];
		$return['master_id'] 	= $master_id;
		$return['master_object_id'] = $master_object_id;
		$return['sub_objects'] 	= $sub_objects;
		return $return;
	}
	return false;
}

/**
 * Fetch an object by name
 * @param $name Name of object to get
 * @param $type Type of object to get
 * @return key/value array with all settings for the object
 */
function getObjectByName($name='', $type='')
{
	$name = addslashes(trim($name));
	$type = addslashes(trim($type));

	if ($name=='' || $type==''){
		echo "Name or type missing when fetching info on object";
		return false;
	}

	switch ($type){
		case 'host':
			$field = 'host_name';
			break;
		case 'service':
			$field = 'service_description';
			break;
	}

	$sql = "SELECT * FROM ".$type." WHERE ".$field."='".$name."'";
	$res = pdo_exec_query($sql);
	$rc = $res->fetch(PDO::FETCH_ASSOC);
	unset($res);
	return $rc;
}

/**
 * Get a list of all saved profiles mathing type
 * @param $type type of profiles to get
 * @return A list of all saved profiles matching type
 */
function getProfilesByType($type='')
{
	$type = trim(addslashes($type));
	if ($type==''){
		return false;
	}
	$classname = $type.'_class';
	require_once('classes/'.$classname.'.php');
	$sql = "SELECT * FROM profiles WHERE object_type='".$type."'";
	$res = pdo_exec_query($sql);
	$ary = $res->fetchAll(PDO::FETCH_ASSOC);
	unset($res);
	$profiles = false;
	foreach($ary as $profile) {
		$obj = new $classname($profile['object_name']);
		if ($obj->can_user_read($type, $obj->id))
			$profiles[] = $profile;
	}
	return $profiles;
}

/**
 * Returns the number of saved subobjects for a profile
 * @param $id The id of the profile for which we should count subobjects
 * @return Number of sub objects
 */
function countProfileSubObjects($id=0)
{
	$id = (int)$id;
	if (!$id){
		return 0;
	}
	$sql = "SELECT COUNT(id)-1 AS cnt FROM profile_objects WHERE profile_id=".$id;
	$res = pdo_exec_query($sql);
        $rc = $res->fetchColumn(0);
        unset($res);
        return (false === $rc) ? 0 : (int)$rc;
}

/**
 * Fetches all profiles by type and prints a select-box
 * and filterlist for easy selection
 * @param $type Type of profiles to fetch
 */
function printProfilesByType($type='')
{
	global $page;
	global $input;
	$type = trim(addslashes($type));
	if ($type==''){
		return false;
	}

	// patch to stop service profiles to show
	if ($type==='service'){
		return false;
	}

	$res = getProfilesByType($type);
	if (!$res || !count($res)){
		echo "<div class='object-table'>No ".$type." profiles available</div>";
		return false;
	}

	// Added som space before table...
	echo '<div class="object-table">Saved profiles of type <strong>'.$type.'</strong></div><div id="object-table">'."\n";

	// table header
	echo "\n<table id=\"profileList\" class=\"ObjTable max zebra\">\n<thead>\n\n";
	echo '<tr class="header">';
	echo "\t<td>Name</td>\n";
	echo "\t<td>Type</td>\n";
	echo "\t<td>Objects</td>\n";
	echo "\t<td>Info</td><td>Actions</td>\n</tr>\n</thead>\n<tbody>\n";

	$i = 0;
	foreach ($res as $row){
		echo "<tr>\n\t<td>".$row['name']."</td>\n";
		echo "\t<td>".htmlentities($row['object_type'])."</td>\n";
		echo "\t<td>".countProfileSubObjects($row['id'])."</td>\n";
		echo "\t<td>".nl2br($row['info'])."</td>\n";
		echo "\t<td style=\"text-align: right; width: 68px\"><a href=\"prototype.php?obj_type=".$type."&amp;action=edit&amp;profile_id=".$row['id']."&amp;obj_type=".$type."&amp;ret_page=".basename($_SERVER['PHP_SELF'])."&amp;obj_id=".$input->obj_id."\"><img src='images/icons/edit.png' alt='Edit' title='Edit' /></a>\n";
		echo "\t&nbsp; <a href=\"prototype.php?obj_type=".$type."&amp;action=delete&amp;profile_id=".$row['id']."\" onclick=\"validateDeleteProfile(".$row['id'].", '".$type."', '".basename($_SERVER['PHP_SELF'])."', ".$input->obj_id.");return false;\"><img src='images/icons/delete.png' alt='Delete' title='Delete' /></a>\n";
		echo "\t&nbsp; <a href=\"prototype.php?obj_type=".$type."&amp;action=use&amp;profile_id=".$row['id']."\"><img src='images/icons/use.png' alt='Use this profile' title='Use this profile' /></a></td>\n";
		echo "</tr>\n";
		$i++;
	}
	echo "</tbody>\n</table>\n";
	echo "</div>";
}

/**
 * Let user input nr of copies/clones to create
 * @param $profile_id Id of profile to use
 * @param $obj_type Type of object the profile refers to
 */
function use_profile_step1($profile_id=0, $obj_type='')
{
	global $page, $obj;

	$profile_id = (int)$profile_id;
	$obj_type 	= trim(addslashes($obj_type));

	if (!$profile_id || $obj_type==''){
		return false;
	}

	$profile = fetchProfile($profile_id);
	//print_r($profile);
	$sub_res = getProfileObjects($profile_id);
	if (!$profile || !$sub_res){
		echo "This profile doesn't seem to be OK - please remove and recreate.<br />";
		echo "<a href=\"#\" onclick=\"history.back();return false;\">back</a><br />";
		return	 false;
	} ?>
	<strong>Edit profile</strong><br />
	Select sub objects to include:
	</div><div id="object-table">
	<form method="post" action="<?php echo $_SERVER['PHP_SELF'] ?>" onsubmit="return checkAndSubmit('prototype', this);" name="prototype_form" id="prototype_form">
	<?php
	if ($profile['master_object_id']){
		$page->hidden_var('host_selection', $profile['master_object_id']);
	}
	$page->hidden_var('obj_id', $profile['master_id']);
	$page->hidden_var('obj_type', $profile['type']);
	?>
	<table class="max zebra ObjTable">
		<tr>
			<td colspan="2"><strong>Name:</strong> <?php echo $profile['name'] ?></td>
		</tr>
		<tr>
			<td colspan="2"><strong>Type:</strong> <?php echo $profile['type'] ?></td>
		</tr>
		<tr>
			<td colspan="2"><strong>Info:</strong> <?php echo nl2br($profile['info']) ?></td>
		</tr>
		<tr>
			<td colspan="2"><strong>Sub objects:</strong> <?php echo countProfileSubObjects($profile_id) ?><br /><br /></td>
		</tr><?php
			foreach($sub_res as $sub_object){
				$sub_type 		= $sub_object['service_description']!='' ? "service" : 'host';
				$sub_fieldname 	= $sub_object['service_description']!='' ? "service_description" : 'host_name';
				$xtra_info 		= translateBitToInfo($sub_object['sub_objects']);

				if ($profile['type']==$sub_type) {
					$ar_name = 'sub_obj';
				} else {
					$ar_name = 'sub_obj['.$sub_type.']['.$profile['sub_objects'][$sub_type][$sub_object[$sub_fieldname]].']';
				} ?>
		<tr class="header">
				<td style="border-right: 0px; padding-left: 7px"><?php if ($profile['type']!=$sub_type) { ?><input type="checkbox" name="sub_obj[<?php echo $sub_type ?>][<?php echo $profile['sub_objects'][$sub_type][$sub_object[$sub_fieldname]] ?>][service]" checked="checked" /><?php } else { ?>&nbsp;<?php } ?></td>
				<td style="border-left: 0px"><?php if ($profile['type']==$sub_type) { ?>Original object: <?php } ?><?php echo $sub_object[$sub_fieldname] ?></td>
		</tr>
			<?php profile_ask_var('Dependencies', $sub_type.'dependency', $ar_name, $xtra_info['dependency']); ?>
			<?php profile_ask_var('Escalations', $sub_type.'escalation', $ar_name, $xtra_info['escalation']); ?>
		<?php
			}?>
	</table><?php

	//$page->hidden_var('obj_type', $obj_type);
	$page->hidden_var('profile_id', $profile_id);

	?>
		<div class="magic max">
			Number of copies to create from this profile: <br />
			<input type="text" size="4" maxlength="3" name="clone_targets" value="1" />
			<input type="button" value="Continue" title="Continue..." class="button continue" id="profileContinueButton" onclick="createProfileDetails(this.form.clone_targets.value, 'profile_details');" />

		</div>

		<div id="profile_details" style="display:none;"></div>
	</form>
	<?php

}

/**
 * Fetch profile sub objects for a profile
 */
function getProfileObjects($profile_id=0)
{
	$profile_id = (int)$profile_id;
	if (!$profile_id){
		echo "Found no profile id - quitting...";
		return false;
	}

	$sql = "SELECT * FROM profile_objects WHERE profile_id=".$profile_id;
	$res = pdo_exec_query($sql);
        $ary = $res->fetchAll(PDO::FETCH_ASSOC);
	unset($res);
        return ($ary && count($ary)) ? $ary : false;
}

/**
 * Print selection list for extended info, dependencies, escalations
 * @param $text Explanatory text
 * @param $vname variable name
 * @param $ar_name array of variable names
 * @param $checked Bool deciding if this should be pre-selected or not
 */
function profile_ask_var($text, $vname, $ar_name, $checked = true)
{
	if ((int)$checked){
		$checked_str = 'checked="checked"';
	} else {
		$checked_str = '';
	}

	echo "<tr>\n<td  class=\"HelpList\" style=\"padding: 2px 7px\">\n";
	echo "  <input type='checkbox' name='{$ar_name}[$vname]' ".$checked_str." /></td><td>$text</td>\n\n";
	echo "\n</tr>\n";
}

/**
 * Delete profile
 * @param $profile_id Id of the profile to delete
 */
function deleteProfile($profile_id=0)
{
	$profile_id = (int)$profile_id;
	if (!$profile_id){
		return false;
	}

	$sql = "DELETE FROM profiles WHERE id=".$profile_id;
	pdo_exec_query($sql);

	$sql = "DELETE FROM profile_objects WHERE profile_id=".$profile_id;
	pdo_exec_query($sql);

	return true;
}

/**
 * Print form for profile editing
 * @param $profile_id id of the profile to print
 */
function profileEditForm($profile_id=0, $ret_page='clone.php', $obj_type='', $obj_id=0)
{
	$profile_id = (int)$profile_id;
	$ret_page	= addslashes(trim($ret_page));
	$obj_type	= addslashes(trim($obj_type));
	$obj_id		= (int)$obj_id;
	$profile 	= false;

	if (!$profile_id){
		return false;
	}

	$sql = "SELECT * FROM profiles WHERE id=".$profile_id;
	$res = pdo_exec_query($sql);
        $profile = $res->fetch(PDO::FETCH_ASSOC);
        unset($res);
        if( ! $profile ) {
		echo "Unable to find the specified profile...";
		return false;
	}

	?>
	<div id="object-table">
	<form method="post" action="<?php echo basename($_SERVER['PHP_SELF']) ?>">
		<div style="margin-left: 1%; margin-top: -10px">
			Name<br>
			<input type="text" name="profile_name" maxlength="100" style="width: 350px" value="<?php echo $profile['name'] ?>" /><br><br>
			Info<br>
			<textarea cols="40" rows="5" name="profile_description" style="width: 350px"><?php echo $profile['info'] ?></textarea><br><br>
			<input type="hidden" name="action" value="update">
			<input type="submit" name="action_dummy" value="Update" title="Update" class="button update" />
			<input type="button" class="button cancel" value="Cancel" title="Cancel" onclick="self.location.href='<?php echo $ret_page ?>?obj_type=<?php echo $obj_type ?>&amp;obj_id=<?php echo $obj_id ?>'" title="Click to return without saving changes..." />
			<input type="hidden" name="obj_id" value="<?php echo $obj_id ?>" />
			<input type="hidden" name="obj_type" value="<?php echo $obj_type ?>" />
			<input type="hidden" name="ret_page" value="<?php echo $ret_page ?>" />
			<input type="hidden" name="profile_id" value="<?php echo $profile_id ?>" />
		</div>
	</form>
	</div>

	<?php
}

/**
 * Update name and description of saved profile
 * @param $profile_id Id of the profile to update
 * @param $profile_name New name of profile
 * @param $profile_description New description of profile
 * @param $obj_type Object type of profile
 */
function updateProfile($profile_id=0, $profile_name='', $profile_description='', $obj_type='')
{
	$profile_id 			= (int)$profile_id;
	$profile_name 			= addslashes(trim($profile_name));
	$profile_description 	= addslashes(trim($profile_description));
	$obj_type				= addslashes(trim($obj_type));

	if (!$profile_id){
		return false;
	}

	if ($profile_name==''){
		$profile_name = ucfirst($obj_type)." profile, ".date('Ymd');
	}

	$sql = "UPDATE profiles SET name='".$profile_name."', info='".$profile_description."' WHERE id=".$profile_id;
	pdo_exec_query($sql);

	return true;

}

/**
 * Update profile to reflect changes to object names
 */
function update_profile_object($obj_type='', $old_name='', $new_name='')
{
	$obj_type	= pdo_quote_string($obj_type);
	$old_name	= pdo_quote_string($old_name);
	$new_name	= pdo_quote_string($new_name);
	$fieldname	= false;

	# safe to always update object_name because we have obj_type
	$sql = "UPDATE profiles SET object_name=$new_name WHERE object_type=$obj_type AND object_name=$old_name";
	pdo_exec_query($sql);

	# Field to update in db differs
	# depending on type of object
	switch ($obj_type) {
		case 'host':
			$fieldname = 'host_name';
			break;
		case 'service':
			$fieldname = 'service_description';
			break;
	}

	if ($fieldname !== false) {
		$query = "UPDATE profile_objects SET $fieldname=$new_name WHERE $fieldname=$old_name";
		pdo_exec_query($query);
	}
	return true;
}
