function helpClass()
{
	var that = this;
	$.getJSON(
		'js/helptexts.json',
		function (data) {
			that.helptexts = data;
		}
	);
}
var authconfigHelper = new helpClass();

function initiateHelptexts(newTab) {
	var selector;
	if (newTab == false) {
		selector = $('.help_icon');
	} else {
		selector = $('#' + newTab + ' .help_icon');
	}
	selector.qtip({
		content: {
			title: {
				text: function() {
					//get the text title attribute for tooltip
					return $(this).closest('tr').find('label').text();
				}
			},
			text: function() {
				var helpContent;
				var heading = $(this).attr('title');
				var lcheading = heading.toLowerCase();
				if (!authconfigHelper.helptexts[lcheading])
					helpContent = 'No help text available';
				else
					helpContent = authconfigHelper.helptexts[lcheading];
				return helpContent;
			}
		},
		position: {
			my: 'top left',
			at: 'bottom right',
			target: 'event',
			//Confine tooltip to window
			viewport: $(window)
		},
		show: {
			//Show when clicked
			event: 'click'
		},
		style: {
			classes: 'ui-tooltip-dark ui-tooltip-rounded'
		}
	});
}

function validate_chars(driver_name) {
	var valid_chars = /[^a-zA-Z0-9]/;
	if (driver_name.match(valid_chars)) {
		alert("ERROR! Invalid character. \nDriver name can only contain alphanumerical characters.");
		return false;
	} else {
		return true;
	}
}

$(document).ready(function() {
	var $driver_name_input = $( "#driver_name"),
		$driver_type_input = $( "#driver_type" );
	
	// tabs init with a custom tab template and an "add" callback filling in the content
	var $tabs = $( "#driver-tabs").tabs({
		tabTemplate: "<li><a href='#{href}'>#{label}</a> <span class='ui-icon ui-icon-close'>#{label}</span></li>",
		add: function( event, ui ) {
			var driver_type = $driver_type_input.val() || "Tab " + driver_type + " content.";
			var driver_name = $driver_name_input.val() || "Tab " + driver_name;

			$.ajax({
				type: 'GET',
				url: 'include/auth_config.inc.php?action=newAuthDriverForm&name=' + driver_name + '&type=' + driver_type,
				dataType: 'html',
				success: function(data) {
					$( ui.panel ).append(data);	
					initiateHelptexts(driver_name);
				},
				error: function(jqXHR, textStatus, errorThrown) {
				}
			});			
			$tabs.tabs('select', '#' + driver_name);
		}
	});

	function validate_driver_name(driver_name) {
		if (!validate_chars(driver_name)) {
			return false;
		}
		var tabNames = Array();
		//Push tab names to array for comparison
		$( "#driver-tabs li" ).each( function() {
			if ($(this).text() !== 'Add New Driver') {
				var name = $(this).text().split(" ");
				tabNames.push(name[0]);
			}			
		});
		//Cannot create driver with the same name as another tab
		if ($.inArray(driver_name, tabNames) > -1) {
			alert("ERROR! Can't create driver. \nYou may not use the same driver name twice.");
			return false;
		//Cannot create driver with empty name
		} else if (driver_name.indexOf(" ") > -1 || driver_name.length == 0) {
			alert("ERROR! Can't create driver. \nDriver name cannot be empty and cannot contain spaces.");
			return false;
		} else {
			return true;
		}
	}

	// modal dialog init: custom buttons and a "close" callback reseting the form inside
	var $dialog = $( "#dialog" ).dialog({
		autoOpen: false,
		modal: true,
		buttons: {
			Add: function() {
				if (validate_driver_name($driver_name_input.val())  === true) {
					addTab();
					$( this ).dialog( "close" );
				} else {
				}
			},
			Cancel: function() {
				$( this ).dialog( "close" );
				$tabs.tabs('select', '#common');
			}
		},
		open: function() {
			$driver_name_input.focus();
			$driver_name_input.keypress(function(e){
				if (e.keyCode  == 13){
					e.preventDefault();
				}
			})
		},
		close: function() {
			$form[ 0 ].reset();
		}
	});

	// addTab form: calls addTab function on submit and closes the dialog
	var $form = $( "form", $dialog ).submit(function() {
		
	});

	// actual addTab function: adds new tab using the title input from the form above
	function addTab() {
		var driver_name = $driver_name_input.val() || "Tab " + driver_name;
		//We need to know how many tabs there are.
		var nextIndex = $( "#driver-tabs li" ).length;
		//Insert tab before add new driver tab.
		$tabs.tabs( "add", "#" + driver_name, driver_name, nextIndex - 1 );
	}

	// addTab button: just opens the dialog
	$( "#new_driver" )
		.click(function() {
			$dialog.dialog( "open" );
		});

	// close icon: removing the tab on click
	// note: closable tabs gonna be an option in the future - see http://dev.jqueryui.com/ticket/3924
	$( "#driver-tabs span.ui-icon-close" ).live( "click", function() {
		var deleteName = $(this).text();
		if (deleteName == 'common')	{
			alert("The common config can't be deleted!");
		} else if ($( "#driver-tabs li" ).length == 3){
			alert("Error! Can't delete driver. \nYou need to have at least one driver configured.");
		} else {
			if (confirm('Do you really want to remove this driver?')) {
				var index = $( "li", $tabs ).index( $( this ).parent() );
				$tabs.tabs( "remove", index );
				$tabs.tabs('select', '#common');
			}
		}
	});

	initiateHelptexts(false);
	$('#create_backup').live('click', function() {
		$.ajax({
			type: 'GET',
			url: 'include/auth_config.inc.php?action=backupConfiguration',
			dataType: 'text',
			success: function(data) {
				$('#backup_status').append(data);
			},
			error: function(jqXHR, textStatus, errorThrown) {
			}
		});
	});
});
