--
-- Procedure copied from comments to https://dev.mysql.com/doc/refman/5.0/en/create-index.html
--
DELIMITER $$
DROP PROCEDURE IF EXISTS `create_index_if_not_exists`$$

CREATE PROCEDURE `create_index_if_not_exists`(table_name_vc varchar(50), index_name_vc varchar(50), field_list_vc varchar(200))
BEGIN

	set @Index_cnt = (
		select count(1) cnt
		FROM INFORMATION_SCHEMA.STATISTICS
		WHERE table_name = table_name_vc
		and index_name = index_name_vc
	);

	IF ifnull(@Index_cnt,0) = 0 THEN set @index_sql = concat('Alter table ',table_name_vc,' ADD INDEX ',index_name_vc,'(',field_list_vc,');');

		PREPARE stmt FROM @index_sql;
		EXECUTE stmt;

		DEALLOCATE PREPARE stmt;

	END IF;

END$$
DELIMITER ;

--
-- Database design for the monitor_webconfig database
--

DROP TABLE IF EXISTS contact_access;
CREATE TABLE contact_access(
	contact			int NOT NULL,
	host			int,
	service			int
);
CREATE INDEX ca_c ON contact_access(contact);
CREATE INDEX ca_cs ON contact_access(contact, service);
CREATE INDEX ca_ch ON contact_access(contact, host);

DROP TABLE IF EXISTS action_log;
CREATE TABLE action_log(
	username			VARCHAR(30) NOT NULL,
	action			VARCHAR(50) NOT NULL,
	time			TIMESTAMP
) ;

DROP TABLE IF EXISTS config_access;
CREATE TABLE config_access(
	username			VARCHAR(30) NOT NULL UNIQUE PRIMARY KEY,
	view_col		BOOL,
	view_all		BOOL,
	modify_obj		BOOL,
	modify_obj_all	BOOL,
	delete_obj		BOOL,
	delete_all		BOOL,
	import			BOOL,
	probe			BOOL,
	admin			BOOL
) COLLATE latin1_general_cs;
INSERT INTO config_access VALUES('monitor', 1, 1, 1, 1, 1, 1, 1, 1, 1);


DROP TABLE IF EXISTS contact;
CREATE TABLE contact(
	id								    INT NOT NULL AUTO_INCREMENT UNIQUE PRIMARY KEY,
	contact_name					    VARCHAR(255),
	alias							    VARCHAR(255) NOT NULL,
	host_notifications_enabled          BOOL,
	service_notifications_enabled       BOOL,
	can_submit_commands                 BOOL,
	retain_status_information           BOOL,
	retain_nonstatus_information        BOOL,
	host_notification_period		    INT,
	service_notification_period		    INT,
	host_notification_options		    VARCHAR(15),
	service_notification_options	    VARCHAR(15),
	host_notification_cmds		    INT,
	host_notification_cmds_args	    TEXT,
	service_notification_cmds       INT,
	service_notification_cmds_args  TEXT,
	email							    VARCHAR(255),
	pager							    VARCHAR(100),
	address1						    VARCHAR(100),
	address2						    VARCHAR(100),
	address3						    VARCHAR(100),
	address4						    VARCHAR(100),
	address5						    VARCHAR(100),
	address6						    VARCHAR(100),
	register							BOOL DEFAULT 1,
	name								VARCHAR(75),
	template						    INT,
	file_id								INT NOT NULL
) COLLATE latin1_general_cs;
CREATE INDEX contact_name ON contact(contact_name);

-- contact, contactgroup:
DROP TABLE IF EXISTS contact_contactgroup;
CREATE TABLE contact_contactgroup(
	contact			INT NOT NULL,
	contactgroup	INT NOT NULL
) ;

DROP TABLE IF EXISTS contactgroup;
CREATE TABLE contactgroup(
	id					INT NOT NULL AUTO_INCREMENT UNIQUE PRIMARY KEY,
	contactgroup_name	VARCHAR(255) NOT NULL,
	alias				VARCHAR(255) NOT NULL,
	register			BOOL DEFAULT 1,
	file_id				INT NOT NULL
) COLLATE latin1_general_cs;
CREATE UNIQUE INDEX contactgroup_name ON contactgroup(contactgroup_name);

DROP TABLE IF EXISTS contactgroup_members;
CREATE TABLE contactgroup_members(
	contactgroup        INT NOT NULL,
	members             INT NOT NULL
);
CREATE INDEX contactgroup_members_group ON contactgroup_members(contactgroup);
CREATE INDEX contactgroup_members_member ON contactgroup_members(members);

DROP TABLE IF EXISTS hostgroup_members;
CREATE TABLE hostgroup_members(
	hostgroup        INT NOT NULL,
	members             INT NOT NULL
);
CREATE INDEX hostgroup_members_group ON hostgroup_members(hostgroup);
CREATE INDEX hostgroup_members_member ON hostgroup_members(members);

DROP TABLE IF EXISTS servicegroup_members;
CREATE TABLE servicegroup_members(
	servicegroup        INT NOT NULL,
	members             INT NOT NULL
);
CREATE INDEX servicegroup_members_group ON servicegroup_members(servicegroup);
CREATE INDEX servicegroup_members_member ON servicegroup_members(members);

DROP TABLE IF EXISTS timeperiod;
CREATE TABLE timeperiod(
	id						INT NOT NULL AUTO_INCREMENT UNIQUE PRIMARY KEY,
	timeperiod_name			VARCHAR(255) NOT NULL,
	alias					VARCHAR(255) NOT NULL,
	sunday					VARCHAR(255),
	monday					VARCHAR(255),
	tuesday					VARCHAR(255),
	wednesday				VARCHAR(255),
	thursday				VARCHAR(255),
	friday					VARCHAR(255),
	saturday				VARCHAR(255),
	register				BOOL DEFAULT 1,
	file_id					INT NOT NULL
) COLLATE latin1_general_cs;
CREATE UNIQUE INDEX timeperiod_name ON timeperiod(timeperiod_name);

-- junction table for timeperiod<->exclude
DROP TABLE IF EXISTS timeperiod_exclude;
CREATE TABLE timeperiod_exclude(
	timeperiod	 INT NOT NULL,
	exclude	 INT NOT NULL
) ;

DROP TABLE IF EXISTS command;
CREATE TABLE command(
	id				INT NOT NULL AUTO_INCREMENT UNIQUE PRIMARY KEY,
	command_name	VARCHAR(255) NOT NULL,
	command_line	BLOB NOT NULL,
	register		BOOL DEFAULT 1,
	file_id			INT NOT NULL
) COLLATE latin1_general_cs;
CREATE UNIQUE INDEX command_name ON command(command_name);


-- host table
DROP TABLE IF EXISTS host;
CREATE TABLE host(
	id								INT NOT NULL AUTO_INCREMENT UNIQUE PRIMARY KEY,
	host_name						VARCHAR(255) DEFAULT NULL,
	alias							VARCHAR(100) NOT NULL,
	display_name                    VARCHAR(100),
	address							VARCHAR(75) NOT NULL,
	initial_state                   VARCHAR(18),
	check_command					INT,
	check_command_args				TEXT,
	max_check_attempts				SMALLINT,
	check_interval					SMALLINT,
	retry_interval                  SMALLINT,
	active_checks_enabled			BOOL,
	passive_checks_enabled			BOOL,
	check_period					INT,
	obsess							BOOL,
	check_freshness					BOOL,
	freshness_threshold				FLOAT,
	event_handler					INT,
	event_handler_args				TEXT,
	event_handler_enabled			BOOL,
	low_flap_threshold				FLOAT,
	high_flap_threshold				FLOAT,
	flap_detection_enabled			BOOL,
	flap_detection_options          VARCHAR(18),
	process_perf_data				BOOL,
	retain_status_information 		BOOL,
	retain_nonstatus_information	BOOL,
	notification_interval			MEDIUMINT,
	first_notification_delay		INT,
	notification_period				INT,
	notification_options			VARCHAR(15),
	notifications_enabled			BOOL,
	stalking_options				VARCHAR(15),
	notes				VARCHAR(255),
	notes_url			VARCHAR(255),
	action_url			VARCHAR(255),
	icon_image			VARCHAR(255),
	icon_image_alt		VARCHAR(255),
	statusmap_image		VARCHAR(255),
	2d_coords			VARCHAR(20),
	name							VARCHAR(75),
	template						INT,
	register						BOOL DEFAULT 1,
	file_id							INT NOT NULL
) COLLATE latin1_general_cs;
CREATE INDEX host_name ON host(host_name);

-- junctions for host objects
DROP TABLE IF EXISTS host_parents;
CREATE TABLE host_parents(
	host	 INT NOT NULL,
	parents	 INT NOT NULL
) ;

DROP TABLE IF EXISTS host_contact;
CREATE TABLE host_contact(
	host	INT NOT NULL,
	contact	INT NOT NULL
) ;

DROP TABLE IF EXISTS host_contactgroup;
CREATE TABLE host_contactgroup(
	host			INT NOT NULL,
	contactgroup	INT NOT NULL
) ;

DROP TABLE IF EXISTS host_hostgroup;
CREATE TABLE host_hostgroup(
	host		INT NOT NULL,
	hostgroup	INT NOT NULL
) ;


DROP TABLE IF EXISTS hostgroup;
CREATE TABLE hostgroup(
	id 		 		   INT NOT NULL UNIQUE PRIMARY KEY,
	hostgroup_name	   VARCHAR(255),
	alias			   VARCHAR(255),
	notes			   VARCHAR(255),
	notes_url		   VARCHAR(255),
	action_url		   VARCHAR(255),
	register		   BOOL DEFAULT 1,
	file_id			   INT NOT NULL
) COLLATE latin1_general_cs;
CREATE UNIQUE INDEX hostgroup_name ON hostgroup(hostgroup_name);


DROP TABLE IF EXISTS service;
CREATE TABLE service(
	id								INT NOT NULL AUTO_INCREMENT UNIQUE PRIMARY KEY,
	host_name						INT DEFAULT NULL,
	hostgroup_name					INT DEFAULT NULL,
	service_description				VARCHAR(255) DEFAULT NULL,
	display_name                    VARCHAR(255),
	is_volatile						BOOL,
	check_command					INT,
	check_command_args				TEXT,
	initial_state                   VARCHAR(1),
	max_check_attempts				SMALLINT,
	check_interval					SMALLINT,
	retry_interval					SMALLINT,
	active_checks_enabled			BOOL,
	passive_checks_enabled			BOOL,
	check_period					INT,
	parallelize_check				BOOL,
	obsess							BOOL,
	check_freshness					BOOL,
	freshness_threshold				INT,
	event_handler					INT,
	event_handler_args				TEXT,
	event_handler_enabled			BOOL,
	low_flap_threshold				FLOAT,
	high_flap_threshold				FLOAT,
	flap_detection_enabled			BOOL,
	flap_detection_options          VARCHAR(18),
	process_perf_data				BOOL,
	retain_status_information		BOOL,
	retain_nonstatus_information	BOOL,
	notification_interval			INT,
	first_notification_delay		INT,
	notification_period				INT,
	notification_options			VARCHAR(15),
	notifications_enabled			BOOL,
	stalking_options				VARCHAR(15),
	notes							VARCHAR(255),
	notes_url						VARCHAR(255),
	action_url						VARCHAR(255),
	icon_image						VARCHAR(255),
	icon_image_alt					VARCHAR(255),
	template						INT,
	name							VARCHAR(75),
	register						BOOL DEFAULT 1,
	file_id							INT NOT NULL
) COLLATE latin1_general_cs;
CREATE INDEX host_service_id ON service(host_name, service_description);
CREATE INDEX hostgroup_service_id ON service(hostgroup_name, service_description);

-- junctions for service objects
DROP TABLE IF EXISTS service_contact;
CREATE TABLE service_contact(
	service	INT NOT NULL,
	contact	INT NOT NULL
) ;

DROP TABLE IF EXISTS service_contactgroup;
CREATE TABLE service_contactgroup(
	service			INT NOT NULL,
	contactgroup	INT NOT NULL
) ;

DROP TABLE IF EXISTS service_servicegroup;
CREATE TABLE service_servicegroup(
	service			INT NOT NULL,
	servicegroup	INT NOT NULL
) ;


DROP TABLE IF EXISTS servicegroup;
CREATE TABLE servicegroup(
	id					INT NOT NULL AUTO_INCREMENT UNIQUE PRIMARY KEY,
	servicegroup_name	VARCHAR(255) NOT NULL,
	alias				VARCHAR(255) NOT NULL,
	notes				VARCHAR(255),
	notes_url			VARCHAR(255),
	action_url			VARCHAR(255),
	register			BOOL DEFAULT 1,
	file_id				INT NOT NULL
) COLLATE latin1_general_cs;
CREATE UNIQUE INDEX servicegroup_name ON servicegroup(servicegroup_name);


DROP TABLE IF EXISTS servicedependency;
CREATE TABLE servicedependency(
	id								INT NOT NULL AUTO_INCREMENT UNIQUE PRIMARY KEY,
	service							INT NOT NULL,
	dependent_service				INT NOT NULL,
	dependency_period				INT,
	inherits_parent					BOOL,
	execution_failure_criteria		VARCHAR(15),
	notification_failure_criteria	VARCHAR(15),
	register						BOOL DEFAULT 1,
	file_id							INT NOT NULL
) COLLATE latin1_general_cs;
CREATE INDEX service ON servicedependency(service);
CREATE INDEX dependent_service ON servicedependency(dependent_service);


DROP TABLE IF EXISTS serviceescalation;
CREATE TABLE serviceescalation(
	id								INT NOT NULL AUTO_INCREMENT UNIQUE PRIMARY KEY,
	service							INT NOT NULL,
	first_notification				MEDIUMINT,
	last_notification				MEDIUMINT,
	notification_interval			MEDIUMINT,
	escalation_period				INT,
	escalation_options				VARCHAR(15),
	register						BOOL DEFAULT 1,
	file_id							INT NOT NULL
) COLLATE latin1_general_cs;

-- junctions for serviceescalation objects
DROP TABLE IF EXISTS serviceescalation_contact;
CREATE TABLE serviceescalation_contact(
	serviceescalation	INT NOT NULL,
	contact				INT NOT NULL
) ;

DROP TABLE IF EXISTS serviceescalation_contactgroup;
CREATE TABLE serviceescalation_contactgroup(
	serviceescalation	INT NOT NULL,
	contactgroup		INT NOT NULL
) ;


DROP TABLE IF EXISTS hostdependency;
CREATE TABLE hostdependency(
	id								INT NOT NULL AUTO_INCREMENT UNIQUE PRIMARY KEY,
	host_name						INT NOT NULL,
	hostgroup_name					INT NOT NULL,
	dependent_host_name				INT NOT NULL,
	dependent_hostgroup_name		INT NOT NULL,
	dependency_period				INT,
	inherits_parent					BOOL,
	execution_failure_criteria		VARCHAR(15),
	notification_failure_criteria	VARCHAR(15),
	register						BOOL DEFAULT 1,
	file_id							INT NOT NULL
) COLLATE latin1_general_cs;

DROP TABLE IF EXISTS hostescalation;
CREATE TABLE hostescalation(
	id								INT NOT NULL AUTO_INCREMENT UNIQUE PRIMARY KEY,
	host_name						INT NOT NULL,
	hostgroup_name					INT NOT NULL,
	first_notification				INT,
	last_notification				INT,
	notification_interval			INT,
	escalation_period				INT,
	escalation_options				VARCHAR(15),
	register						BOOL DEFAULT 1,
	file_id							INT NOT NULL
) COLLATE latin1_general_cs;


-- junctions for hostescalation objects
DROP TABLE IF EXISTS hostescalation_contact;
CREATE TABLE hostescalation_contact(
	hostescalation	INT NOT NULL,
	contact			INT NOT NULL
) ;

DROP TABLE IF EXISTS hostescalation_contactgroup;
CREATE TABLE hostescalation_contactgroup(
	hostescalation		INT NOT NULL,
	contactgroup		INT NOT NULL
) ;


-- custom variables
DROP TABLE IF EXISTS custom_vars;
CREATE TABLE custom_vars(
	obj_type	VARCHAR(30) NOT NULL,
	obj_id		INT NOT NULL,
	variable	VARCHAR(255),
	value		VARCHAR(255)
) COLLATE latin1_general_cs;
-- No single object can have multiple variables named the same
CREATE UNIQUE INDEX objvar ON custom_vars(obj_type, obj_id, variable);

-- webconfig <=> gui db scheme cross-pollination ends here

DROP TABLE IF EXISTS hostextinfo;
CREATE TABLE hostextinfo(
	id					INT NOT NULL UNIQUE PRIMARY KEY,
	host_name			INT NOT NULL,
	notes				VARCHAR(255),
	notes_url			VARCHAR(255),
	action_url			VARCHAR(255),
	icon_image			VARCHAR(255),
	icon_image_alt		VARCHAR(255),
	statusmap_image		VARCHAR(255),
	2d_coords			VARCHAR(20),
	file_id				INT NOT NULL
) COLLATE latin1_general_cs;
CREATE UNIQUE INDEX host_name ON hostextinfo(host_name);

DROP TABLE IF EXISTS serviceextinfo;
CREATE TABLE serviceextinfo(
	id								INT NOT NULL UNIQUE PRIMARY KEY,
	service							INT NOT NULL,
	notes							VARCHAR(255),
	notes_url						VARCHAR(255),
	action_url						VARCHAR(255),
	icon_image						VARCHAR(255),
	icon_image_alt					VARCHAR(255),
	file_id							INT NOT NULL
) COLLATE latin1_general_cs;
CREATE UNIQUE INDEX service ON serviceextinfo(service);


-- configuration files
-- This table has been renamed, so drop the old one if it exists
DROP TABLE IF EXISTS FILE;
DROP TABLE IF EXISTS FILE_tbl;
DROP TABLE IF EXISTS file_tbl;
CREATE TABLE file_tbl(
	id            INT NOT NULL AUTO_INCREMENT UNIQUE PRIMARY KEY,
	file_name     VARCHAR(255) NOT NULL
) ;
CREATE UNIQUE INDEX file_name ON file_tbl(file_name);

--
-- Structure for table 'profiles'
--

CREATE TABLE IF NOT EXISTS profiles (
  id int(11) NOT NULL auto_increment,
  name varchar(255) NOT NULL,
  object_type varchar(25) NOT NULL,
  object_name varchar(255) NOT NULL,
  info text,
  PRIMARY KEY  (id),
  KEY object_type (object_type),
  KEY object_name (object_name)
) COLLATE latin1_general_cs;

--
--  Structure for table 'profile_objects'
--

CREATE TABLE IF NOT EXISTS profile_objects (
  id int(11) NOT NULL auto_increment,
  profile_id int(11) NOT NULL,
  host_name varchar(255) default NULL,
  service_description VARCHAR(255) default NULL,
  sub_objects int(11) NOT NULL default '0',
  master_obj_id int(11) NOT NULL default '0',
  PRIMARY KEY  (id),
  KEY sub_objects (sub_objects),
  KEY prototype_id (profile_id)
) COLLATE latin1_general_cs;

--
-- Structure for table `changelog`
--

DROP TABLE IF EXISTS changelog;
CREATE TABLE IF NOT EXISTS changelog (
  id int(11) NOT NULL auto_increment,
  object_type varchar(100) NOT NULL,
  object_id int(11) NOT NULL,
  oldname varchar(255) DEFAULT NULL,
  newname varchar(255) DEFAULT NULL,
  oldstate mediumblob DEFAULT NULL,
  newstate mediumblob DEFAULT NULL,
  `username` varchar(255) default NULL,
  `time` timestamp NULL default CURRENT_TIMESTAMP,
  PRIMARY KEY (id),
  KEY object_type (object_type)
) COLLATE latin1_general_cs;

call create_index_if_not_exists('changelog', 'chlg_username_idx', 'username');

--
-- Structure for table `changelog_history`
--

CREATE TABLE IF NOT EXISTS changelog_history (
  id int(11) NOT NULL auto_increment,
  object_type varchar(100) NOT NULL,
  object_name varchar(255) NOT NULL,
  oldname varchar(255) DEFAULT NULL,
  newname varchar(255) DEFAULT NULL,
  oldstate mediumblob DEFAULT NULL,
  newstate mediumblob DEFAULT NULL,
  `username` varchar(255) default NULL,
  `time` timestamp NULL default CURRENT_TIMESTAMP,
  PRIMARY KEY (id),
  KEY object_type (object_type)
) COLLATE latin1_general_cs;

call create_index_if_not_exists('changelog_history', 'chlghst_username_idx', 'username');

--
-- Structure for table `objlocks`
--

DROP TABLE IF EXISTS objlocks;
CREATE TABLE IF NOT EXISTS objlocks (
  id int(11) NOT NULL auto_increment,
  object_type varchar(100) NOT NULL,
  object_id int(11) NOT NULL,
  username varchar(255) DEFAULT NULL,
  time int(11) NOT NULL,
  PRIMARY KEY (id),
  KEY object_type (object_type),
  KEY object_id (object_id)
) COLLATE latin1_general_cs;


-- obsoleted tables
DROP TABLE IF EXISTS tmp;

